//===========================================================================
//
//      EditFile.cpp - Script Editor File Functions
//
//      Copyright 1993-2003 Cypress Software Inc.  All Rights Reserved.
//      Information Contained Herein Is Proprietary and Confidential.
//	
//===========================================================================

#include "stdafx.h"

//===========================================================================
static void FileError( HWND hwnd, char *szMessage, char *szText )
//===========================================================================
{
    char szBuffer[64 + _MAX_PATH];
    wsprintf( szBuffer, szMessage, szText );
    DisplayMessage( hwnd, szBuffer );
}

//===========================================================================
static char szFileName[_MAX_PATH];
static int nCurLine = -1;

//===========================================================================
static void UpdateCaption( HWND hwnd )
//===========================================================================
{
	// Get the title portion of the file name 
	char szTitleName[_MAX_PATH];
	if ( *szFileName )
		GetFileTitle( szFileName, szTitleName, sizeof(szTitleName) );
	else
		lstrcpy( szTitleName, _T("Untitled") );

	// Generate the caption string
    char szCaption[64 + _MAX_PATH];

	lstrcpy( szCaption, szTitleName );
	lstrcat( szCaption, _T(" - ") );
	lstrcat( szCaption, pAppTitle );

	if ( nCurLine > -1 ) {
		int nLen = lstrlen( szCaption );
		if ( nLen < _MAX_PATH )
			wsprintf( &szCaption[nLen], _T(" (Stopped at line: %d)"), nCurLine );
	}

	// Set the window caption
    SetWindowText( hwnd, szCaption );
}

//===========================================================================
static char szFilter[] = _T("Basic Files (*.bas)\0*.bas\0")
                         _T("All Files (*.*)\0*.*\0\0")
						 _T("");

static OPENFILENAME ofn = {
    sizeof(OPENFILENAME),
    0,
    NULL,
    szFilter,
    NULL,
    0,
    0,
    NULL,          // Set in Open and Close functions
    _MAX_PATH,
    NULL,          // Set in Open and Close functions
    0,
    NULL,
    NULL,
    0,             // Set in Open and Close functions
    0,
    0,
    "bas",
    0L,
    NULL,
    NULL,
};

//===========================================================================
static BOOL EditFileOpenDlg( HWND hwnd, PSTR pstrFileName )
//===========================================================================
{
    ofn.hwndOwner         = hwnd;
    ofn.lpstrFile         = pstrFileName;
    ofn.Flags             = OFN_HIDEREADONLY | OFN_CREATEPROMPT;

    return GetOpenFileName( &ofn );
}

//===========================================================================
static BOOL EditFileSaveDlg( HWND hwnd, PSTR pstrFileName )
//===========================================================================
{
    ofn.hwndOwner         = hwnd;
    ofn.lpstrFile         = pstrFileName;
    ofn.Flags             = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;

    return GetSaveFileName( &ofn );
}

#ifndef SEM_LOADFILE
#define SEM_LOADFILE			0x00EB
#define SEM_SAVEFILE			0x00EA
#endif

//===========================================================================
static BOOL EditFileRead( HWND hwndEdit, PSTR pstrFileName, HWND hwnd )
//===========================================================================
{
	// Check for empty filename parameter
    if ( pstrFileName == NULL || *pstrFileName == '\0' )
		return FALSE;

	// If currently debugging, then close out the session
	enaRunMenuEnd( hwndEdit );

	// Load hourglass cursor
	HCURSOR hCursor = SetCursor( LoadCursor(NULL,IDC_WAIT) );
	ShowCursor( TRUE );

	// Read text from file and color the text
	BOOL bFileLoaded = !SendMessage( hwndEdit, SEM_LOADFILE, (WPARAM)pstrFileName, 0 );

	// Restore original cursor
	ShowCursor( FALSE );
	SetCursor( hCursor );

	// If the file loaded successfully, then update caption
	if ( bFileLoaded ) {
		if ( pstrFileName != szFileName )
			strncpy( szFileName, pstrFileName, sizeof(szFileName) );
		UpdateCaption( hwnd );

		return TRUE;
	}
	
	FileError( hwnd, "Could not read file %s!", pstrFileName );

    return FALSE;
}

//===========================================================================
static HRESULT EditFileWrite( HWND hwndEdit, PSTR pstrFileName, HWND hwnd )
//===========================================================================
{
	// Write text to the file
	BOOL bFileSaved = !SendMessage( hwndEdit, SEM_SAVEFILE, (WPARAM)pstrFileName, 0 );

	// If the file was saved successfully, then update caption
	if ( bFileSaved ) {
		SendMessage( hwndEdit, EM_SETMODIFY, 0, 0 );
		UpdateCaption( hwnd );
		return S_OK;
	}
	
	FileError( hwndEdit, "Could not write file %s", pstrFileName );
				
	return E_FAIL;
}

//===========================================================================
STDAPI QuerySaveFile( HWND hwndEdit, HWND hwnd )
//===========================================================================
{
	BOOL bNeedSave = SendMessage( hwndEdit, EM_GETMODIFY, 0, 0 );
	if ( ! bNeedSave )
		return IDNO;

	char szTitleName[_MAX_PATH];
	if ( *szFileName )
		GetFileTitle( szFileName, szTitleName, sizeof(szTitleName) );
	else
		lstrcpy( szTitleName, _T("Untitled") );

    char szBuffer[64 + _MAX_PATH];
    wsprintf( szBuffer, "Save changes to %s?", szTitleName );
    int iReturn = MessageBox( hwndEdit, szBuffer, pAppTitle, MB_YESNOCANCEL | MB_ICONQUESTION );
    if ( iReturn == IDYES && EditFileWrite(hwndEdit,szFileName,hwnd ) )
        iReturn = IDCANCEL;

	return iReturn;
}

//===========================================================================
STDAPI enaFileMenuNew( HWND hwndEdit, HWND hwnd )
//===========================================================================
{
	if ( IDCANCEL == QuerySaveFile(hwndEdit,hwnd) )
		return 0;

	// If currently debugging, then close out the session
	enaRunMenuEnd( hwndEdit );

	SetWindowText( hwndEdit, "\0" );
	szFileName[0]  = '\0';
	UpdateCaption( hwnd );

	return S_OK;
}

//===========================================================================
STDAPI enaFileMenuOpen( HWND hwndEdit, HWND hwnd )
//===========================================================================
{
	static BOOL bInOpenDlg;

	if ( bInOpenDlg )
		return S_OK;

	bInOpenDlg = TRUE;

	BOOL bFileRead = FALSE;
	if ( QuerySaveFile(hwndEdit,hwnd) != IDCANCEL )
		if ( EditFileOpenDlg(hwnd, szFileName) )
			bFileRead = EditFileRead( hwndEdit, szFileName, hwnd );

	bInOpenDlg = FALSE;

	return S_OK;
}

//===========================================================================
STDAPI enaFileMenuSaveAs( HWND hwndEdit, HWND hwnd )
//===========================================================================
{
	if ( EditFileSaveDlg(hwnd, szFileName) )
		return EditFileWrite( hwndEdit, szFileName, hwnd );

	return S_OK;
}

//===========================================================================
STDAPI enaFileMenuSave( HWND hwndEdit, HWND hwnd )
//===========================================================================
{
	if ( szFileName[0] )
		return EditFileWrite( hwndEdit, szFileName, hwnd );

	return enaFileMenuSaveAs( hwndEdit, hwnd );
}

//=============================================================================================
extern "C" void CALLBACK HideDebugCursor( HANDLE hCtx, HWND hWnd )
//=============================================================================================
{
	nCurLine = -1;
	UpdateCaption( GetParent(hWnd) );
}

//=============================================================================================
extern "C" void CALLBACK ShowDebugCursor( HANDLE hCtx, HWND hWnd, int bufNum, long nLine )
//=============================================================================================
{
	if ( bufNum == 2 ) {
		nCurLine = nLine;
		UpdateCaption( GetParent(hWnd) );
	}
}
