//===========================================================================
//
//      RunDebug.cpp - Script Editor
//
//      Copyright 1993-2003 Cypress Software Inc.  All Rights Reserved.
//      Information Contained Herein Is Proprietary and Confidential.
//
//===========================================================================

#include "stdafx.h"

#include "AppFuncs.h"

#ifndef ENS_DEBUG_BREAK
#define ENS_DEBUG_BREAK ENS_HIT_BREAKPOINT
#endif

//=============================================================================================
HANDLE	g_ctx;	// Used by SDIEdit.cpp to initialize menus (Start/Continue)
VARIANT	g_vRetVar = {0};
LPTSTR	g_szFunctionName = NULL;	// _T("Main");

static VARIANT MyVar = {2,0,0,0,42};


//============================================================================
HRESULT CALLBACK Output( LPENOUTPUTINFOA pOutputInfo )
//============================================================================
{
	// This callback function duplicates the built in functionality.
	// Optionally output could be written to a window or logged to a file.

	if ( pOutputInfo->type == 3 )	// Runtime error
		return EN_DISPLAYOUTPUT;	// Display the runtime error dialog box

	// Print statements are displayed without an icon in the message box
	MessageBox( 0, pOutputInfo->errMessage, pAppTitle
		, pOutputInfo->type ? MB_ICONWARNING : MB_OK );

	return EN_DEFAULTACTION;
}

//=============================================================================================
extern "C" void CALLBACK HideDebugCursor( HANDLE hCtx, HWND hWnd );
extern "C" void CALLBACK ShowDebugCursor( HANDLE hCtx, HWND hWnd, int bufNum, long nLine );

//=============================================================================================
static HRESULT CompileScript( HWND hwndEdit, HANDLE &hCtx )
//=============================================================================================
{
	if ( hCtx )
		return 0;

    HRESULT hr = enaCreateContextA( pAppTitle, 0, hwndEdit, &hCtx );

	if ( SUCCEEDED(hr) ) {
		long lOption = ENOPT_CALL_APP_WITH_CTX	// Implicit context as first parameter
//			| ENOPT_ALLOW_IMMEDIATE_CODE		// Allow immediate code,
//			| ENOPT_NO_IMPLICIT_END_STATEMENT;	// But emulate Enable 3.2 behavior
			;
		enaSetOption( hCtx, lOption, TRUE );
		enaAppendText( hCtx, pDeclarations );
		hr = enaAppendEditWindow( hCtx, hwndEdit );
	}

	if ( FAILED(hr) ) {
		TCHAR *pMsgBuf = 0;
		DWORD dwCount = FormatMessage( 
			FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, NULL, hr, 
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (TCHAR *)&pMsgBuf, 0, NULL );
		MessageBox( hwndEdit, dwCount ? pMsgBuf : _T("Error running script"), pAppTitle, MB_OK );
		LocalFree( pMsgBuf );
	}
	else {
		enaRegisterOutputCallback( hCtx, Output );
		enaRegisterGetProcAddrCallback( hCtx, GetAppProcAddr );
		hr = enaCompile( hCtx );	// Compile errors are reported by Enable
	}

	if ( FAILED(hr) ) {
		enaFreeContext( hCtx );
		hCtx = 0;
	}

	SendMessage( hwndEdit, SEM_SETCONTEXT, (WPARAM)hCtx, 0 );

	// Add code here to set global variables, etc.

	{	// Use local HRESULT to avoid modifying the actual return value
	static char sX[256];
	strcpy( sX, "abc" );
	HRESULT hr = enaSetGlobalFixedStringRef( hCtx, "sX", sX );		// Returns E_FAIL

	static WCHAR wsX[256];
	wcscpy( wsX, L"abc" );
	hr = enaSetGlobalFixedStringRefW( hCtx, L"wsX", wsX );

	//VariantClear( &MyVar );
	HRESULT rc = enaSetGlobalVariantRef( hCtx, "MyVar", &MyVar );
	}

	return hr;
}

//=============================================================================================
STDAPI enaDebugMenuStepInto( HWND hwndEdit )
//=============================================================================================
{
	HRESULT hr;

	if ( g_ctx )
		hr = enaDebugStepInto( g_ctx );
	else {
		hr = CompileScript( hwndEdit, g_ctx );
		if ( SUCCEEDED(hr) ) {
			if ( g_szFunctionName )
				hr = enaDebugFunction( g_ctx, g_szFunctionName, NULL, &g_vRetVar );
			else
				hr = enaDebugStepInto( g_ctx );
		}
	}

	if ( hr != ENS_DEBUG_BREAK )
		enaRunMenuEnd( hwndEdit );

	return hr;
}

//=============================================================================================
STDAPI enaDebugMenuStepOver( HWND hwndEdit )
//=============================================================================================
{
	HRESULT hr;

	if ( g_ctx == 0 ) {
		hr = CompileScript( hwndEdit, g_ctx );
		if ( SUCCEEDED(hr) && g_szFunctionName ) {
			hr = enaCallFunction( g_ctx, g_szFunctionName, NULL, &g_vRetVar );
			if ( hr != ENS_DEBUG_BREAK )
				enaRunMenuEnd( hwndEdit );
			return hr;
		}
	}
	if ( g_ctx )
		hr = enaDebugStep( g_ctx );

	if ( hr != ENS_DEBUG_BREAK )
		enaRunMenuEnd( hwndEdit );

	return hr;
}

//=============================================================================================
STDAPI enaRunMenuStart( HWND hwndEdit )
//=============================================================================================
{
	HRESULT hr;

	if ( g_ctx )
		hr = enaDebugGo( g_ctx  );
	else {
		hr = CompileScript( hwndEdit, g_ctx );
		if ( SUCCEEDED(hr) ) {
			if ( g_szFunctionName )
				hr = enaCallFunction( g_ctx, g_szFunctionName, NULL, &g_vRetVar );
			else
				hr = enaExecute( g_ctx );
		}
	}

	if ( hr != ENS_DEBUG_BREAK )
		enaRunMenuEnd( hwndEdit );

    return hr;
}

//=============================================================================================
STDAPI enaRunMenuEnd( HWND hwndEdit )
//=============================================================================================
{
	if ( g_ctx ) {
		// Add code here to get global variables, etc.
		enaFreeContext( g_ctx );
		g_ctx = 0;
	}
	return S_OK;
}

//=============================================================================================
STDAPI enaRunMenuDebugFunction( HWND hwndEdit )
//=============================================================================================
{
	HRESULT hr = S_OK;
//	g_szFunctionName = _T("Main");

	if ( g_ctx == 0 )
		hr = CompileScript( hwndEdit, g_ctx );

	if ( SUCCEEDED(hr) )
		hr = enaDebugFunction( g_ctx, /*g_szFunctionName*/_T("Main"), NULL, &g_vRetVar );

//	if ( hr != ENS_DEBUG_BREAK )
//		enaRunMenuEnd( hwndEdit );

	return hr;
}
