//===========================================================================
//
//      SDIEdit.cpp : Defines the entry point for the application
//
//      Copyright 1993-2003 Cypress Software Inc.  All Rights Reserved.
//      Information Contained Herein Is Proprietary and Confidential.
//	
//===========================================================================

#include "stdafx.h"
#include <richedit.h>

#include "resource.h"

//===========================================================================
// Global Variables
HWND hDlgModeless;

//===========================================================================
int APIENTRY WinMain( HINSTANCE hInst, HINSTANCE, PSTR pCmdLine, int nCmdShow )
//===========================================================================
{
	// Register the window class
	LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
	WNDCLASSEX wcex = {sizeof(WNDCLASSEX),CS_HREDRAW|CS_VREDRAW,WndProc};
	wcex.hInstance		= hInst;
	wcex.hIcon			= LoadIcon( hInst, (LPCTSTR)IDI_SDIEDIT );
	wcex.hCursor		= LoadCursor( NULL, IDC_ARROW );
	wcex.lpszMenuName	= (LPCTSTR)IDC_SDIEDIT;
	wcex.lpszClassName	= pAppTitle;

	RegisterClassEx( &wcex );

	// Initialize the script edit control
	HRESULT hr = enaInitEnableEdit( 0 );
	if ( FAILED(hr) ) {
		PTSTR errStr;
		FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
			NULL, hr, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (PTSTR)&errStr, 0, NULL );
		MessageBox( NULL, errStr, pAppTitle, MB_OK | MB_ICONWARNING );
		LocalFree( errStr );
		return FALSE;
	}

	// Create frame window and its script edit child window
	HWND hWnd = CreateWindow( pAppTitle, pAppTitle, WS_OVERLAPPEDWINDOW,
				CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInst, NULL );
	if ( !hWnd )
		return FALSE;

	ShowWindow( hWnd, nCmdShow );
	UpdateWindow( hWnd );

	HACCEL hAccelTable = LoadAccelerators( hInst, (LPCTSTR)IDC_SDIEDIT );

	// Main message loop:
	MSG msg;
	while ( GetMessage( &msg, NULL, 0, 0 ) ) {
        if ( hDlgModeless == NULL || !IsDialogMessage(hDlgModeless, &msg) ) {
            if ( !TranslateAccelerator(hWnd, hAccelTable, &msg) ) {
				TranslateMessage( &msg );
				DispatchMessage( &msg );
			}
		}
	}

	return msg.wParam;
}

//===========================================================================
// Handle to client window
static HWND hwndEdit;

// Message handlers:
//===========================================================================
static LRESULT wmCreate( HWND hWnd, LPARAM lParam )
//===========================================================================
{
	// Create the edit control child window
	CREATESTRUCT *pCreateStruct = (CREATESTRUCT *)lParam;
	hwndEdit = CreateWindowEx ( WS_EX_CLIENTEDGE, /*pcszEnableEditClassName*/"EnableEdit42", NULL
		, WS_CHILD | WS_VISIBLE | ES_MULTILINE | WS_HSCROLL | WS_VSCROLL |
			ES_AUTOHSCROLL | ES_AUTOVSCROLL | ES_DISABLENOSCROLL |
			ES_NOHIDESEL | ES_SAVESEL | ES_SELECTIONBAR,
		0, 0, 0, 0, hWnd, (HMENU)1, pCreateStruct->hInstance, NULL );

	if ( hwndEdit == NULL )
		return -1;

	// We want selection change notification for Ln, Col,  
//	SendMessage( EM_SETEVENTMASK, 0, ENM_CHANGE | ENM_SELCHANGE );

	// Load type library for script tips and syntax coloring
	ITypeLib *pTypeLib;
	HRESULT hr = LoadTypeLib( L"SDIEdit.exe", &pTypeLib );
	if ( SUCCEEDED(hr) ) {
		SendMessage( hwndEdit, SEM_ADDTYPELIBREF, (WPARAM)pTypeLib, 0 );
		pTypeLib->Release();
	}

	// Set maximum file size to 1 megabyte and font to fixed
	SendMessage( hwndEdit, EM_LIMITTEXT, 0x100000, 0);
	SendMessage( hwndEdit, WM_SETFONT, (WPARAM)GetStockObject(ANSI_FIXED_FONT), 0);

	DWORD dwColorTable [] = {
		-9999997,		// Auto   - normal script text color
		0xff0000,		// Blue   - keywords
		0x008000,		// Green  - comments
		0x008080,		// Olive  - built in functions
		-9999997,		// Auto   - identifiers
		0x800080,		// Purple - numbers
		0x0000ff,		// Red    - strings
		0x000080		// DkRed  - constants
	};
	SendMessage( hwndEdit, SEM_SETCOLORTABLE, sizeof(dwColorTable), (LPARAM)dwColorTable );

	SendMessage( hwndEdit, SEM_SETEDITOPTIONS, 1, 0 );	// Color while editing

	enaFileMenuNew( hwndEdit, hWnd );

	return 0;
}

//===========================================================================
static LRESULT wmSize( HWND hWnd, LPARAM lParam )
//===========================================================================
{
	int cx = LOWORD( lParam );
	int cy = HIWORD( lParam );

	// Adjust client window size
	MoveWindow( hwndEdit, 0, 0, cx, cy, TRUE );
	
	return 0;
}

//===========================================================================
inline LRESULT wmSetFocus()
//===========================================================================
{
	// This sets the focus to the edit control when the editor gets focus
	SetFocus( hwndEdit );
	return 0;
}

//===========================================================================
inline LRESULT wmActivateApp( WPARAM wParam )
//===========================================================================
{
	// This hides the selection when a another application is active
	SendMessage( hwndEdit, EM_HIDESELECTION, (LOWORD(wParam)==WA_INACTIVE), 0 );
	return 0;
}

//===========================================================================
static LRESULT wmInitMenuPopup( HWND hwnd, WPARAM wParam, LPARAM lParam )
//===========================================================================
{
	HMENU hMenu = (HMENU)wParam;

	switch ( lParam ) {
	case 1:				// Edit menu
		{
		// Enable Undo if edit control can do it
		EnableMenuItem( hMenu, IDM_UNDO,
			SendMessage(hwndEdit, EM_CANUNDO, 0, 0L) ? MF_ENABLED : MF_GRAYED );

		// Enable Paste if text is in the clipboard
		EnableMenuItem(  hMenu, IDM_PASTE,
			IsClipboardFormatAvailable(CF_TEXT) ? MF_ENABLED : MF_GRAYED );

		// Enable Cut, Copy, and Del if text is selected
		int iSelBeg, iSelEnd;
		SendMessage( hwndEdit, EM_GETSEL, (WPARAM)&iSelBeg, (LPARAM)&iSelEnd );
		int iEnable = iSelBeg != iSelEnd ? MF_ENABLED : MF_GRAYED;
		EnableMenuItem( hMenu, IDM_CUT,   iEnable );
		EnableMenuItem( hMenu, IDM_COPY,  iEnable );
		EnableMenuItem( hMenu, IDM_CLEAR, iEnable );

        // Enable Find, Next, and Replace if modeless dialog is not already active
        iEnable = hDlgModeless == NULL ? MF_ENABLED : MF_GRAYED;
        EnableMenuItem( hMenu, IDM_FIND,    iEnable );
        EnableMenuItem( hMenu, IDM_NEXT,    iEnable );
        EnableMenuItem( hMenu, IDM_REPLACE, iEnable );
        break;
		}
	case 3:				// Run menu
		{
		extern HANDLE g_ctx;
		char *cp = g_ctx ? "&Continue\tF5" : "&Start\tF5";
		ModifyMenu( hMenu, IDM_START, MF_BYCOMMAND | MF_STRING , IDM_START, cp );
		}
    }
    return 0;
}

//===========================================================================
LRESULT CALLBACK About(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
//===========================================================================
{
	switch ( uMsg ) {
	case WM_INITDIALOG:
			return TRUE;

	case WM_COMMAND:
		if ( LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL ) {
			EndDialog( hDlg, LOWORD(wParam) );
			return TRUE;
		}
		break;
	}
    return FALSE;
}

HRESULT CALLBACK EnumBreakPointsProc(
  ENBREAKPOINT *pBreakpoint,	// Pointer to read-only breakpoint structure
  LPARAM lParam					// Application-defined value
)
{
	if ( lParam ) {
		HRESULT hr = enaSetBreakpoint( (HANDLE)lParam, pBreakpoint->nBuf, pBreakpoint->nLine );
		if ( FAILED(hr) )
			MessageBox( 0, _T("Invalid breakpoint"), pAppTitle, MB_OK );
	}
	return S_OK;
}

static LRESULT wmClose( HWND hWnd );
//===========================================================================
static LRESULT wmCommand( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
//===========================================================================
{
	extern HANDLE g_ctx;

	// Handle menu commands
	if ( lParam == 0 ) switch ( LOWORD(wParam) ) {

	// Messages from File menu
	case IDM_NEW:		return enaFileMenuNew(	  hwndEdit, hWnd );
	case IDM_OPEN:		return enaFileMenuOpen(	  hwndEdit, hWnd );
	case IDM_SAVE:		return enaFileMenuSave(	  hwndEdit, hWnd );
	case IDM_SAVEAS:	return enaFileMenuSaveAs( hwndEdit, hWnd );
	case IDM_EXIT:		return wmClose(hWnd);

	// Messages from Edit menu
	case IDM_UNDO:		return SendMessage( hwndEdit, WM_UNDO,	 0,  0 );
	case IDM_CUT:		return SendMessage( hwndEdit, WM_CUT,	 0,  0 );
	case IDM_COPY:		return SendMessage( hwndEdit, WM_COPY,	 0,  0 );
	case IDM_PASTE:		return SendMessage( hwndEdit, WM_PASTE,	 0,  0 );
	case IDM_CLEAR:		return SendMessage( hwndEdit, WM_CLEAR,  0,  0 );
	case IDM_SELALL:	return SendMessage( hwndEdit, EM_SETSEL, 0, -1);

	case IDM_FIND:		return enaEditMenuFind(		hwndEdit, hWnd );
	case IDM_NEXT:		return enaEditMenuFindNext( hwndEdit );
	case IDM_REPLACE:	return enaEditMenuReplace(	hWnd );

	// Messages from Debug menu
	case IDM_STEPINTO:		return enaDebugMenuStepInto( hwndEdit );
	case IDM_STEPOVER:		return enaDebugMenuStepOver( hwndEdit );
	case IDM_BREAKPOINT:	return SendMessage( hwndEdit, SEM_BREAKPOINT,	0, 0 );
	case IDM_CLEARBRKPNTS:	return SendMessage( hwndEdit, SEM_CLEARBRKPNTS,	0, 0 );
	case IDM_ENUMBRKPNTS:	return SendMessage( hwndEdit, SEM_ENUMBREAKPOINTS, (WPARAM)EnumBreakPointsProc, (LPARAM)g_ctx );

	// Messages from Run menu
	case IDM_START:			return enaRunMenuStart( hwndEdit );
	case IDM_END:			return enaRunMenuEnd(   hwndEdit );
	case IDM_DEBUGGO:		return enaDebugGo( g_ctx );
	case IDM_DEBUGRESET:	return enaDebugReset( g_ctx );
	case IDM_DEBUGFUNCTION:	return enaRunMenuDebugFunction( hwndEdit );

	// Messages from Help menu
	case IDM_HELP:
		WinHelp( hWnd, "Enable", HELP_FINDER, 0 );
		return 0;
	case IDM_ABOUT:
		{
		HINSTANCE hInst = (HINSTANCE)GetWindowLong(hWnd,GWL_HINSTANCE);
		DialogBox(hInst, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)About);
		return 0;
		}
	}
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

//===========================================================================
static LRESULT wmQueryEndSession( HWND hWnd )
//===========================================================================
{
	return IDCANCEL != QuerySaveFile(hwndEdit,hWnd);
}

//===========================================================================
static LRESULT wmClose( HWND hWnd )
//===========================================================================
{
	if ( wmQueryEndSession( hWnd ) )
		DestroyWindow( hWnd );

	return 0;
}

//===========================================================================
inline LRESULT wmDestroy()
//===========================================================================
{
	PostQuitMessage( 0 );
	return 0;
}

//===========================================================================
//
//  FUNCTION: WndProc(HWND, unsigned, WORD, LONG)
//
//  PURPOSE:  Processes messages for the main window.
//
//  WM_CREATE			- Create the edit control child window
//  WM_SIZE				- Adjust client window size
//	WM_SETFOCUS:		- Sets the focus to the edit control
//	WM_ACTIVATEAPP:		- Show/hide selection
//	WM_INITMENUPOPUP:	- Sets menu state
//  WM_COMMAND			- Process the application menu
//  WM_CLOSE			- Save changes on close window?
//  WM_QUERYENDSESSION	- Save changes on system shutdown?
//  WM_DESTROY			- Post a quit message and return
//
//
//===========================================================================
LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
//===========================================================================
{
	static UINT iMsgFindReplace = RegisterWindowMessage( FINDMSGSTRING );

	switch ( uMsg ) {
	case WM_CREATE:			return wmCreate(		hWnd,				lParam );
	case WM_SIZE:			return wmSize(			hWnd,				lParam );
	case WM_SETFOCUS:		return wmSetFocus();
	case WM_ACTIVATEAPP:	return wmActivateApp(				wParam		   );
	case WM_INITMENUPOPUP:	return wmInitMenuPopup(	hWnd,		wParam, lParam );
	case WM_COMMAND:		return wmCommand(		hWnd, uMsg, wParam, lParam );
	case WM_CLOSE:			return wmClose(			hWnd );
	case WM_QUERYENDSESSION:return wmQueryEndSession(hWnd);
	case WM_DESTROY:		return wmDestroy();
	default:
 		if ( uMsg == iMsgFindReplace )
 			return HandleFindReplace( hwndEdit, (FINDREPLACE *)lParam );
	}
	return DefWindowProc( hWnd, uMsg, wParam, lParam );
}
